import { ASTNode, ASTNodeTemplate, NumberNode } from '3h-ast';
export declare const HELP_SYMBOL: unique symbol;
export declare const CONSTRUCTOR_SYMBOL: unique symbol;
export declare const BASE_SYMBOL: unique symbol;
export declare const PROMISE_SYMBOL: unique symbol;
export declare type ContextValue = null | boolean | number | string | Dict | FunctionHandler | ContextValue[];
export interface Dict {
    [key: string]: ContextValue;
    [HELP_SYMBOL]?: string;
    [CONSTRUCTOR_SYMBOL]?: ContextValue;
    [BASE_SYMBOL]?: ContextValue;
    [PROMISE_SYMBOL]?: Promise<ContextValue>;
}
export declare type ValueNode = ASTNodeTemplate<'value', {
    value: ContextValue;
}>;
export declare type SyntaxNode = ASTNode | ValueNode;
/**
 * absolutePath -> exports
 */
export declare type ResolvedModules = Record<string, Dict>;
export interface ScriptContext {
    store: Dict;
    exports: Dict;
    resolvedModules: ResolvedModules;
    source: string;
    basePath: string;
    /**
     * [old, ..., new]
     */
    stack: string[];
}
export declare type SyntaxHandler = (buffer: SyntaxNode[], index: number, context: ScriptContext) => void;
export interface FunctionHandler<T extends ContextValue = ContextValue> {
    (rawArgs: readonly SyntaxNode[], referrer: SyntaxNode, context: ScriptContext, thisArg: ContextValue): T;
    [HELP_SYMBOL]?: string;
}
export declare namespace Utils {
    /**
     * A stable in-place sorting function.
     */
    const sort: <T>(array: T[], compare: (a: T, b: T) => number) => void;
    /**
     * Remove specific element(s) from the array.
     * (Default count: `array.length - start`)
     */
    const removeElements: (array: any[], start: number, count?: number) => void;
    const normalizeIndex: (index: number, arraySize: number, referrer: SyntaxNode, context: ScriptContext, allowEnd?: boolean) => number;
    const formatFrameString: (referrer: SyntaxNode, context: ScriptContext) => string;
    /**
     * Throw a specific error with environment info appended.
     */
    const raise: (constructor: ErrorConstructor, message: string, referrer: SyntaxNode | null, context: ScriptContext | null) => never;
    const parseNumber: (node: NumberNode, context: ScriptContext) => number;
    const isDict: (value: unknown) => value is Dict;
    const createDict: (dict: Dict) => Dict;
    const createValueNode: (value: ContextValue, referrer: SyntaxNode) => ValueNode;
    const createRawArray: (array: ContextValue[], referrer: SyntaxNode) => SyntaxNode[];
    const replaceBuffer: (buffer: SyntaxNode[], start: number, width: number, replacement: SyntaxNode) => void;
    /**
     * Excute the callback with the temporary variables
     * defined in context store and recover conflicting
     * variables after execution.
     */
    const injectTemp: (store: Dict, variables: Dict, callback: () => void) => void;
    const extendContext: (context: ScriptContext, store: Dict, stack: string[]) => ScriptContext;
    const injectHelp: <T>(helpInfo: string, target: T) => T;
    const toString: (value: ContextValue, previewSize?: number, previewIndent?: string) => string;
    const toDisplay: (value: ContextValue, previewSize?: number, previewIndent?: string) => string;
    const filterValue: (value: unknown) => ContextValue;
}
